@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2010-2013 Innocent De Marchi <tangram.peces@gmail.com>
+ * Copyright (C) 2010-2016 Innocent De Marchi <tangram.peces@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  ***********************************************************************/
-#include <QtGui>
+//#include <QtGui>
+#include <QtWidgets/QGraphicsSceneMouseEvent>
 
 
 #include "fitxa.h"
@@ -105,10 +106,10 @@ void Fitxa::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,  QW
 
 /*
  if (p_aillada){
-    painter->setPen(QPen(Qt::yellow, 4) );
-  //   gradientLinear.setColorAt(1, QColor(Qt::red));
-  //     painter->setBrush(gradientLinear);
- }*/
+    painter->setBrush(Qt::black );
+ }
+ */
+
 
  if (m_estat==6){
    painter->setBrush(Qt::NoBrush);
@@ -302,7 +303,7 @@ void Fitxa::calculaMovimentsPosiblesFitxa(){
                                                   (coord+m_coordenades).x()*100+(coord+m_coordenades).y(),
                                                   (coord*2+m_coordenades).x()*100+(coord*2+m_coordenades).y() ));
                 }
-                //estan permesos els movimens en diagonal
+                //estan permesos els moviments en diagonal
                 else if (p_tipusMoviment>=3){//posam l'index de la fitxa botada i de la fitxa final
                     movimentsPossibles.append(QPoint(
                                                   (coord+m_coordenades).x()*100+(coord+m_coordenades).y(),
@@ -372,7 +373,7 @@ void Fitxa::setEstat( int nouValor , bool canviarFitxaFinalJoc)
  // if( (m_estat != nouValor) || (nouValor==0) ){
       m_estat = nouValor;
      //Si la fitxa es "mou", llavors reinciam l'aïllament
-     if(m_estat==0) {setFixaAillada(false);}
+     if(m_estat==0) {setFitxaAillada(false);}
      update();//}
 }
 
@@ -449,7 +450,7 @@ void Fitxa::setFitxaSolitariPersonalitzat(bool valor){
     else qDebug("S'Ha marcat una fitxa com a NO personalitzada");*/
 }
 
-void Fitxa::setFixaAillada(bool valor){
+void Fitxa::setFitxaAillada(bool valor){
     if(p_aillada != valor){
         p_aillada=valor;
         p_nivellAillament=0;
