@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2010-2013 Innocent De Marchi <tangram.peces@gmail.com>
+ * Copyright (C) 2010-2016 Innocent De Marchi <tangram.peces@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ class Tauler;
 
 class Fitxa : public QGraphicsEllipseItem
 {
+    Q_GADGET
 public:
     Fitxa( Tauler*  taulerJoc, QGraphicsItem* parent=0);
 
@@ -50,6 +51,8 @@ public:
 
    void calculaMovimentsPosiblesFitxa();
 
+   //Conserva les coordenades de les fitxes destí de la fitxa actual
+   QList<QPoint> movimentsFitxa;
    void calculaMovimentsFitxa();
 
    bool fitxaFinalJoc() const{
@@ -104,7 +107,7 @@ public:
 
     void setEsMou(bool valor);
 
-    void setFixaAillada(bool valor);
+    void setFitxaAillada(bool valor);
 
     void setNivellAillament( int valor);
 
@@ -123,7 +126,7 @@ protected:
     virtual void mousePressEvent(QGraphicsSceneMouseEvent* event);
 
 private:
-   //Estat de la fitza per pintar-la
+   //Estat de la fitxa per pintar-la
    int m_estat;
 
    //Coordenades de la fitxa en el joc
@@ -162,8 +165,6 @@ private:
    //generació de solitari personalitzat
    bool p_fitxaSolitariPersonalitzat;
 
-   //Conserva les coordenades de les fitxes destí de la fitxa actual
-   QList<QPoint> movimentsFitxa;
 
    //Tauler de joc al qual pertany la fitxa
    Tauler* m_taulerJoc;
