@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2010-2013 Innocent De Marchi <tangram.peces@gmail.com>
+ * Copyright (C) 2010-2016 Innocent De Marchi <tangram.peces@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,14 @@
 #define FRMPRINCIPAL_H
 
 #include <QtGui>
+#include <QtWidgets/QTreeWidget>
+#include <QtWidgets/QTreeWidgetItem>
+#include <QtWidgets/QTabWidget>
+#include <QtWidgets/QUndoStack>
+#include <QtWidgets/QMenu>
+#include <QtWidgets/QActionGroup>
+#include <QtWidgets/QToolBar>
+#include <QtWidgets/QMainWindow>
 #include <QTranslator>
 
 #include "frmprincipal.h"
@@ -191,19 +199,26 @@ private slots:
     void canviaIdioma(QAction *action);
 
     void creditsProgramaSlot();
+    void webProgramaSlot();
+    void webProgramaPecesSlot();
 
     void ajudaSlot();
     void agraimentSlot();
 
+    /*Procediments de cerca de solició*/
+    void solucionaJocRapid();
+    bool comprovaExisteixenMovimentsPossibles();
+    void solucionaJocActual_carregaSolucio();
+    void solucionaJocActual_carregaSolucio1();
     void solucionaJocActual_Inicia();
     void solucionaJocActual_CarregaMovimentsInicialsArbre();
     void solucionaJocActual_comencaRecerca();
     void solucionaJocActual_eliminaMovimentsPosteriors();
     void solucionaJocActual_eliminaConfiguracionsPosteriors(QString config);
+    void solucionaJocActual_RecercaIterativa();
     //Aquesta funció no es fa servir
    // void solucionaJocActual_eliminaConfiguracions();
 
-    void solucionaJocRapid();
 
     void desaConfiguracionsEliminades(int tipus=0);
     void carregaConfiguracionsEliminades();
@@ -341,6 +356,8 @@ private:
     QAction *colaboraPrograma;
     QAction *creditsPrograma;
     QAction *agraiment;
+    QAction *webProgramaAction;
+    QAction *webProgramaPecesAction;
 
     QAction *solucionaJocActual;
     QAction *solucionaJocActualRapid;
@@ -367,6 +384,8 @@ private:
     //rellotge per generar els nombre aleatoris
     QTime rellotgeQTime;
 
+    void actualitzaCodisArxiusDeSolucio();
+
     protected:
 
       void resizeEvent ( QResizeEvent * event );
