@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2010-2013 Innocent De Marchi <tangram.peces@gmail.com>
+ * Copyright (C) 2010-2016 Innocent De Marchi <tangram.peces@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,5 +31,8 @@ QString directoriLinux();
   */
 int numeroConfiguracio(QString config);
 
+/* fa una pausa en milisegons
+void pausa(int temps);*/
+
 
 #endif // FUNCIONS_H
