@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2010-2013 Innocent De Marchi <tangram.peces@gmail.com>
+ * Copyright (C) 2010-2016 Innocent De Marchi <tangram.peces@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -96,7 +96,8 @@ void Moviment::fesMoviment(int primera, int segona, int tercera){
 }
 
 void Moviment::eliminaFitxesVermelles(){
-    QHashIterator <int, Fitxa*> i(p_taulerJoc->m_fitxes);
+    //QHashIterator <int, Fitxa*> i(p_taulerJoc->m_fitxes);
+    QMapIterator <int, Fitxa*> i(p_taulerJoc->m_fitxes);
     while (i.hasNext()) {
         i.next();
         if (i.value()->estat()==5){
@@ -111,7 +112,8 @@ void Moviment::eliminaFitxesVermelles(){
  * "marcat"
  */
 void Moviment::eliminaMovimentMarcat(){
-    QHashIterator <int, Fitxa*> i(p_taulerJoc->m_fitxes);
+    //QHashIterator <int, Fitxa*> i(p_taulerJoc->m_fitxes);
+    QMapIterator <int, Fitxa*> i(p_taulerJoc->m_fitxes);
     while (i.hasNext()) {
         i.next();
       /* Tipus de moviment normal i diagonal
