@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2010-2013 Innocent De Marchi <tangram.peces@gmail.com>
+ * Copyright (C) 2010-2016 Innocent De Marchi <tangram.peces@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -91,7 +91,8 @@ void Tauler::marcaMoviment (QString moviment){
   Elimina els buits marcats com a final d'un moviment anterior
   */
 void Tauler::eliminaMarquesMoviments(const  QPoint& coordenades){
-    QHashIterator <int, Fitxa*> i(m_fitxes);
+    //QHashIterator <int, Fitxa*> i(m_fitxes);
+    QMapIterator <int, Fitxa*> i(m_fitxes);
     while (i.hasNext()) {
         i.next();
         if (i.value()->estat()==3){
@@ -123,7 +124,8 @@ void Tauler::ferMoviment (int fitxaInicial, int fitxaBotada, int fitxaFinal,bool
 
 void Tauler::marcaFinalDeJoc(int direccioMoviment, bool marcaFinal){
     if( jocFinalitzat()){
-        QHashIterator <int, Fitxa*> i(m_fitxes);
+        //QHashIterator <int, Fitxa*> i(m_fitxes);
+        QMapIterator <int, Fitxa*> i(m_fitxes);
         int numFitxes=0, ordFitxa=0;
         int numFitxesFinalJoc=0;
         jocResolt=1;
@@ -216,7 +218,8 @@ void Tauler::marcaFinalDeJoc(int direccioMoviment, bool marcaFinal){
  * de joc (i estan activades!)
  */
 bool Tauler::nomesQuedenFitxesMarcadesFinalJoc(){
-  QHashIterator <int, Fitxa*> i(m_fitxes);
+  //QHashIterator <int, Fitxa*> i(m_fitxes);
+  QMapIterator <int, Fitxa*> i(m_fitxes);
   bool nomesFinalJoc=true;
   while (i.hasNext()) {
         i.next();
@@ -244,7 +247,8 @@ bool Tauler::nomesQuedenFitxesMarcadesFinalJoc(){
   */
 bool Tauler::jocFinalitzat(){
 
-   QHashIterator <int, Fitxa*> i(m_fitxes);
+   //QHashIterator <int, Fitxa*> i(m_fitxes);
+   QMapIterator <int, Fitxa*> i(m_fitxes);
    //Pressuposam que el joc ha acabat
    bool finalitzat=true;
    while (i.hasNext()) {
@@ -319,6 +323,9 @@ void Tauler::joc_CarregaJoc( QString p_tipusJoc, QString p_filesColumnes, QStrin
     comptadorFitxesEstatZeroJoc=0;
     p_estatDelJoc=joc;
 
+/*qCritical("p_tipusJoc %s", qUtf8Printable(p_tipusJoc));
+qCritical("p_filesColumnes %s", qUtf8Printable(p_filesColumnes));
+qCritical("p_estatFitxes %s", qUtf8Printable(p_estatFitxes));*/
 
     QStringList  tipusJoc = p_tipusJoc.split(" ");
     setModalitatJocActual(tipusJoc.value(0));
@@ -340,15 +347,21 @@ void Tauler::joc_CarregaJoc( QString p_tipusJoc, QString p_filesColumnes, QStrin
         for(int j=0 ; j<files;++j){
               for(int i=0 ; i<columnes;++i){
                    if(estatFitxes.value(j*columnes+i).toInt()>-1){
-                      QPoint coord(j,i); //fila i columna
+                       QPoint coord(j,i); //fila i columna
                        Fitxa *fitxa= new Fitxa(this);
                        fitxa->setCoordenades(coord);
                        fitxa->setEstat(estatFitxes.value(j*columnes+i).toInt());
+                      // qDebug("%d",estatFitxes.value(j*columnes+i).toInt());
+                       /*qCritical("%s", qPrintable(QString("Contador %1 estatFitxa %2")
+                                                  .arg(estatFitxes.value(j*columnes+i).toInt())
+                                                  .arg(estatFitxes.value(j*columnes+i).toInt())));*/
                        //fitxa->setFitxaSolitariPersonalitzat(false);
                        fitxa->setTipusMoviment(tipusJoc.value(1).toInt());
                        fitxa->setPos(fitxa->coordenades().y()*distanciaEntreFitxes*30,
                                              fitxa->coordenades().x()*distanciaEntreFitxes*30);
                        m_fitxes.insert(coord.x()*100+coord.y(),fitxa);
+                       //qDebug("Insertada fitxa %d",coord.x()*100+coord.y());
+                       //configuracioActual(false);
                        scene()->addItem(fitxa);
                        if (fitxa->estat()==1) {++comptadorFitxesJoc;}
                        if (fitxa->estat()==0){++comptadorFitxesEstatZeroJoc;}
@@ -385,6 +398,7 @@ void Tauler::joc_CarregaJoc( QString p_tipusJoc, QString p_filesColumnes, QStrin
            }//for files
     }
 p_movimentsUndoStack->clear();
+configuracioActual(false);
 setInteractive(true);
 //Això per a què s'ajusti a la finestra
 fitInView(sceneRect(), Qt::KeepAspectRatio);
@@ -412,7 +426,8 @@ bool Tauler::ocuparFitxesFinalJoc(){
   bool resultat=true;
   /* Si és un joc amb fitxes marcades com a final del joc
    */
-  QHashIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
+  //QHashIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
+  QMapIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
   bool ambFitxesFinalJoc=false;
   while (fitxesFinalsJoc.hasNext()) {
        fitxesFinalsJoc.next();
@@ -431,7 +446,8 @@ bool Tauler::ocuparFitxesFinalJoc(){
  * del joc que estan sense ocupar
  */
 int Tauler::numeroFitxesFinalsSenseOcupar(){
-    QHashIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
+    //QHashIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
+    QMapIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
     int numeroFitxesFinalJoc=0;
     while (fitxesFinalsJoc.hasNext()) {
          fitxesFinalsJoc.next();
@@ -448,7 +464,8 @@ int Tauler::numeroFitxesFinalsSenseOcupar(){
  * a final de joc)
  */
 int Tauler::numeroFitxesPerEliminar(){
-    QHashIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
+    //QHashIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
+    QMapIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
     int numeroFitxesFinalJoc=0;
     while (fitxesFinalsJoc.hasNext()) {
          fitxesFinalsJoc.next();
@@ -472,14 +489,14 @@ int Tauler::numeroFitxesPerEliminar(){
  void Tauler::solucionaJocActual_CalculaMovimentsPosibles(){
       //Per a cada fitxa del joc, calcula els possibles moviments
      //i es desen a la QList fitxa.movimentsPossibles
-     QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+     //QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+     QMapIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
      while (iterator_fitxes.hasNext()) {
         iterator_fitxes.next();
         iterator_fitxes.value()->calculaMovimentsPosiblesFitxa();
      }
  }
 
-
  /* Comprova els moviments possibles, els selecciona segons diversos
   * criteris i preferències, i retorna una llista amb els moviments
   * preferits
@@ -487,7 +504,8 @@ int Tauler::numeroFitxesPerEliminar(){
  QStringList Tauler::solucionaJocActual_CarregaMovimentsArbre
                      (QStringList movimentsFets, QString darrerMoviment){
 
-    QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+    //QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+    QMapIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
     QStringList itemsArbre; //tots els moviments possibles
     QStringList itemsArbre1;//moviments que enllaçan amb l'anterior
     QStringList itemsArbre2;//moviments per preferència
@@ -496,13 +514,15 @@ int Tauler::numeroFitxesPerEliminar(){
     QStringList itemsArbre5;//moviments pel número de moviments posteriors
     QStringList itemsArbre6;//moviments que cumpleixen almenys 3 requisits
 
-  //  solucionaJocActual_FitxesAillades();
-    int numeroMovimentsInicialsAtzar=3;
+    solucionaJocActual_FitxesAillades1();
+    /* Número de moviments que es faran al principi a l'atzar*/
+    int numeroMovimentsInicialsAtzar=qrand() % comptadorFitxesJoc;
+   // int numeroMovimentsInicialsAtzar=round(comptadorFitxesJoc*0.4);
+    //qDebug("numeroMovimentsInicialsAtzar %d", numeroMovimentsInicialsAtzar);
     int preferenciaMenor=200;
     int aillamentMajor=0;
       while (iterator_fitxes.hasNext()) {
         iterator_fitxes.next();
-
         int index_fitxaInicial=
             iterator_fitxes.value()->coordenades().x()*100+
             iterator_fitxes.value()->coordenades().y();
@@ -527,8 +547,13 @@ int Tauler::numeroFitxesPerEliminar(){
                  // && (solucionaJocActual_CalculaMovimentsPosteriors(moviment))
                    ){
                 itemsArbre.append(moviment);
-                if (p_movimentsUndoStack->index()>numeroMovimentsInicialsAtzar+10){return itemsArbre;}
                 ++movimentsPosibles;
+                /*Això s'ha posat una mica més abaix.
+                  La intenció és aconseguir més diversitat en els primers moviments
+                  del joc */
+                /*if (p_movimentsUndoStack->index()<numeroMovimentsInicialsAtzar){
+                    qDebug("Condició itemsArbre.count() %d", itemsArbre.count());
+                    return itemsArbre;}*/
                 }
                }
            } //for
@@ -539,6 +564,13 @@ int Tauler::numeroFitxesPerEliminar(){
          }
         }//if inicial estat==1
       }
+
+      /* Si no hi ha moviments possibles, no val la pena perdre el temps!*/
+      if(itemsArbre.count()<1){return itemsArbre;}
+      /* Per als primers moviments del joc, no es seleccionan tant els moviments */
+      if (p_movimentsUndoStack->index()<numeroMovimentsInicialsAtzar){
+          return itemsArbre;}
+
       //Miram quins moviments maximitzen el número de moviments posteriors
       itemsArbre5=solucionaJocActual_CalculaNumeroMovimentsPosteriors(itemsArbre);
 
@@ -559,7 +591,7 @@ int Tauler::numeroFitxesPerEliminar(){
              itemsArbre1.append(itemsArbre.value(j));
            //  if (p_movimentsUndoStack->index()>numeroMovimentsInicialsAtzar ){return itemsArbre1;}
 
-             if (m_fitxes.value(fitxesMovimentArbre.value(0).toInt())->preferenciaMoure()==preferenciaMenor ){
+             if (m_fitxes.value(fitxesMovimentArbre.value(0).toInt())->preferenciaMoure()==preferenciaMenor){
                 itemsArbre3.prepend(itemsArbre.value(j));
               //  if (p_movimentsUndoStack->index()>numeroMovimentsInicialsAtzar ){return itemsArbre3;}
                }
@@ -576,40 +608,70 @@ int Tauler::numeroFitxesPerEliminar(){
        }//for
     }// if fitxesDarrerMoviment.count()
 
-//return itemsArbre
-for(int j=0 ; j<itemsArbre5.count();++j){
-    int repeticio=0;
-    if (itemsArbre1.contains(itemsArbre5.value(j))) {++repeticio;}
-    if (itemsArbre2.contains(itemsArbre5.value(j))) {++repeticio;}
-    if (itemsArbre3.contains(itemsArbre5.value(j))) {++repeticio;}
-    if (itemsArbre4.contains(itemsArbre5.value(j))) {++repeticio;}
-    if (repeticio>2){itemsArbre6.append(itemsArbre5.value(j));}
-}
 
+for(int j=0 ; j<itemsArbre.count();++j){
+    int repeticio=0;
+    if (itemsArbre1.contains(itemsArbre.value(j))) {++repeticio;}
+    if (itemsArbre2.contains(itemsArbre.value(j))) {++repeticio;}
+    if (itemsArbre3.contains(itemsArbre.value(j))) {++repeticio;}
+    if (itemsArbre4.contains(itemsArbre.value(j))) {++repeticio;}
+    if (itemsArbre5.contains(itemsArbre.value(j))) {++repeticio;}
+    if (repeticio>2){itemsArbre6.append(itemsArbre.value(j));}
+}
+//qDebug("itemsArbre6.count() %d", itemsArbre6.count());
 QStringList movimentsSelecionats;
-/*      // moviments que cumpleixen més de 3 requisits
+/*  versió 2.0 moviments que cumpleixen més de 3 requisits
       if (itemsArbre6.count()>0){return itemsArbre6;}
+      //moviments per nivell d'aïllament de les fitxes
+      else if (itemsArbre4.count()>0){return itemsArbre4;}
+      //moviments per preferència que enllaçan amb l'anterior
       else if (itemsArbre3.count()>0){return itemsArbre3;}
-      //Maximitzar el número de moviments
+      //moviments pel número de moviments posteriors
       else  if (itemsArbre5.count()>0){return itemsArbre5;}
-      // nomès enllaçar moviment
+      //moviments que enllaçan amb l'anterior
       else  if (itemsArbre1.count()>0){return itemsArbre1;}
-      // preferència en el moviment
+      //moviments per preferència
       else if (itemsArbre2.count()>0){return itemsArbre2;}
       else return itemsArbre;*/
 
-// moviments que cumpleixen més de 3 requisits
-if (itemsArbre6.count()>0){movimentsSelecionats.append(itemsArbre6);}
-else if (itemsArbre3.count()>0){movimentsSelecionats.append(itemsArbre3);}
-//Maximitzar el número de moviments
-else  if (itemsArbre5.count()>0){movimentsSelecionats.append(itemsArbre5);}
-// nomès enllaçar moviment
-else  if (itemsArbre1.count()>0){movimentsSelecionats.append(itemsArbre1);}
-// preferència en el moviment
+/*
+if (itemsArbre5.count()>0){movimentsSelecionats.append(itemsArbre5);}
 else if (itemsArbre2.count()>0){movimentsSelecionats.append(itemsArbre2);}
+else if (itemsArbre6.count()>0){movimentsSelecionats.append(itemsArbre6);}
+//else  if (itemsArbre6.count()>0){movimentsSelecionats.append(itemsArbre6);}
+//else  if (itemsArbre2.count()>0){movimentsSelecionats.append(itemsArbre2);}
+//else if (itemsArbre1.count()>0){movimentsSelecionats.append(itemsArbre1);}
 else movimentsSelecionats.append(itemsArbre);
+*/
 
-//Ara comprova si hi ha algun moviment que te com a final un fitxa
+/*
+if (itemsArbre6.count()>0){return itemsArbre6;}
+else if (itemsArbre4.count()>0){return itemsArbre4;}
+else if (itemsArbre5.count()>0){return itemsArbre5;}
+else if (itemsArbre2.count()>0){return itemsArbre2;}
+else if (itemsArbre3.count()>0){return itemsArbre3;}
+else  if (itemsArbre1.count()>0){return itemsArbre1;}
+else movimentsSelecionats.append(itemsArbre);
+*/
+
+/*15/09/16 No està clar quina opció és millor*/
+movimentsSelecionats.append(itemsArbre5);
+movimentsSelecionats.append(itemsArbre2);
+movimentsSelecionats.append(itemsArbre6);
+if (movimentsSelecionats.count()<1){movimentsSelecionats.append(itemsArbre);}
+
+
+/*
+    QStringList itemsArbre; //tots els moviments possibles
+    QStringList itemsArbre1;//moviments que enllaçan amb l'anterior
+    QStringList itemsArbre2;//moviments per preferència
+    QStringList itemsArbre3;//moviments per preferència que enllaçan amb l'anterior
+    QStringList itemsArbre4;//moviments per nivell d'aïllament de les fitxes
+    QStringList itemsArbre5;//moviments pel número de moviments posteriors
+    QStringList itemsArbre6;//moviments que cumpleixen almenys 3 requisits    
+*/
+
+//Ara comprova si hi ha algun moviment que te com a final una fitxa
 //marcada com a final de joc
 if(comptadorFitxesFinalJoc>0){
    // qDebug("movimentsSelecionats.count() %d",movimentsSelecionats.size());
@@ -631,7 +693,17 @@ if(comptadorFitxesFinalJoc>0){
  }
  //else qDebug("no hi ha moviments finals");
 }
+/*
+ qDebug("itemsArbre1.count() %d", itemsArbre1.count());
+ qDebug("itemsArbre2.count() %d", itemsArbre2.count());
+ qDebug("itemsArbre3.count() %d", itemsArbre3.count());
+ qDebug("itemsArbre4.count() %d", itemsArbre4.count());
+ qDebug("itemsArbre5.count() %d", itemsArbre5.count());
+ qDebug("itemsArbre6.count() %d", itemsArbre6.count());
+ qDebug("movimentsSelecionats.count() %d", movimentsSelecionats.count());
+ qDebug("*****************");*/
  return movimentsSelecionats;
+// return itemsArbre;
 }
 
  /*
@@ -640,9 +712,8 @@ if(comptadorFitxesFinalJoc>0){
    Retorna la clau del Has de la fitxa amb més moviments
    */
  QStringList Tauler::solucionaJocActual_CalculaNumeroMovimentsPosteriors(QStringList movimentsPosibles){
-     int maxNumMoviments=0;
-     int numeroMoviments=0;
-     QStringList llistaMovimentsMaxims;
+     //qDebug("solucionaJocActual_CalculaNumeroMovimentsPosteriors");     
+     QList <int> llistaNumeroMoviments;
      for(int j=0 ; j<movimentsPosibles.count();++j){
          //Primer "simulam" el moviment
          QStringList moviment=QStringList(movimentsPosibles.value(j).split(" "));
@@ -650,13 +721,16 @@ if(comptadorFitxesFinalJoc>0){
              m_fitxes.value(moviment.value(0).toInt())->setEstat(0);
              m_fitxes.value(moviment.value(1).toInt())->setEstat(0);
              m_fitxes.value(moviment.value(2).toInt())->setEstat(1);
-             //Ara calculam el número de moviments possibles amb aquesta situació
-             numeroMoviments=solucionaJocActual_MovimentsPosiblesActuals();
-             if (numeroMoviments>maxNumMoviments){
-                 // numeroMovimentMaxim=j;
+             /* 09/09/16 Canviat
+              * Ara calculam el número de moviments possibles amb aquesta situació
+              */
+             llistaNumeroMoviments.append(solucionaJocActual_MovimentsPosiblesActuals());
+             /*if (numeroMoviments>maxNumMoviments){
                  maxNumMoviments=numeroMoviments;
+                 if(llistaMovimentsMaxims.count()>0){
+                 llistaMovimentsMaxims.removeLast();}
                  llistaMovimentsMaxims.append(movimentsPosibles.value(j));
-             }
+             }*/
              //desfem el moviment
              m_fitxes.value(moviment.value(0).toInt())->setEstat(1);
              m_fitxes.value(moviment.value(1).toInt())->setEstat(1);
@@ -667,27 +741,58 @@ if(comptadorFitxesFinalJoc>0){
              m_fitxes.value(moviment.value(1).toInt())->setEstat(1);
              m_fitxes.value(moviment.value(2).toInt())->setEstat(1);
              //Ara calculam el número de moviments possibles amb aquesta situació
-             numeroMoviments=solucionaJocActual_MovimentsPosiblesActuals();
-             if (numeroMoviments>maxNumMoviments){
-                 // numeroMovimentMaxim=j;
+             llistaNumeroMoviments.append(solucionaJocActual_MovimentsPosiblesActuals());
+            /* if (numeroMoviments>maxNumMoviments){
                  maxNumMoviments=numeroMoviments;
+                 if(llistaMovimentsMaxims.count()>0){
+                 llistaMovimentsMaxims.removeLast();}
                  llistaMovimentsMaxims.append(movimentsPosibles.value(j));
-             }
+             }*/
              //desfem el moviment
              m_fitxes.value(moviment.value(0).toInt())->setEstat(1);
              m_fitxes.value(moviment.value(1).toInt())->setEstat(0);
              m_fitxes.value(moviment.value(2).toInt())->setEstat(0);
          }//else p_tipusMoviment != 2
      }//for
-  return llistaMovimentsMaxims;
+
+  return solucionaJocActual_SeleccionaNumeroMovimentsPosteriors(movimentsPosibles,llistaNumeroMoviments);
+
+  //qDebug("llistaMovimentsMaxims.count() %d", llistaMovimentsMaxims.count());
+  //return llistaMovimentsMaxims;
  }
 
+/* Dels moviments possibles, selecciona els que tenen el major nombre de moviments
+ * posteriors
+ */
+QStringList Tauler::solucionaJocActual_SeleccionaNumeroMovimentsPosteriors(QStringList llistaMovimentsMaxims,
+                                                                           QList <int> llistaNumeroMoviments){
+  /* Màxim nombre de moviments*/
+  int maximNombreDeMoviments=0;
+  for(int j=0 ; j<llistaNumeroMoviments.count();++j){
+     if(llistaNumeroMoviments.value(j)>maximNombreDeMoviments){
+       maximNombreDeMoviments=llistaNumeroMoviments.value(j);
+      // qDebug("maximNombreDeMoviments %d", maximNombreDeMoviments);
+     }
+
+  }
+  /* Seleccionam els moviments que maximitzam el nombre de moviments posteriors*/
+  QStringList movimentsMaxims;
+  for(int j=0 ; j<llistaNumeroMoviments.count();++j){
+     if(llistaNumeroMoviments.value(j)==maximNombreDeMoviments){
+       movimentsMaxims.append(llistaMovimentsMaxims.value(j));
+     }
+  }
+ //qDebug("movimentsMaxims.count() %d", movimentsMaxims.count());
+ return movimentsMaxims;
+}
+
  /* Comprova si s'ha arribat al final del joc. Això
   * és molt útil en les modalitats que tenen el final
   * de joc marcat
   */
 bool Tauler::arribatFinalJoc(){
-QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+//QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+QMapIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
 int numeroFitxesActives=0;
 while (iterator_fitxes.hasNext()) {
    iterator_fitxes.next();
@@ -744,8 +849,11 @@ bool finalDeJoc=false;
  }
  */
 
+/* Calcula el nombre de moviments reals de la configuració actual
+ */
  int Tauler::solucionaJocActual_MovimentsPosiblesActuals(){ 
-    QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+    QMapIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+   // QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
 
     int movimentsPosibles=0;
     int numeroFitxesActives=0;
@@ -777,57 +885,226 @@ if (arribatFinalJoc()){movimentsPosibles=qMax(movimentsPosibles,1);}
    que es pugui fer efectiu o be cap moviment en què
    la fitxa mitjancera pugui passar a estat 1 desprès d'un moviment
    18/11/12 aques proc. no pareix molt eficaç
-   */
+   03/09/16 Substituït pel procediement solucionaJocActual_FitxesAillades1()
+   a la versió 2.1
+   *
  bool Tauler::solucionaJocActual_FitxesAillades(){
-     /* Aquest procediment no és eficient
-      * amb la modalitat inversa
-      */
+     qDebug("Inici aillades*************");
+     // Aquest procediment no és eficient
+     // amb la modalitat inversa
      if(p_tipusMoviment==2){return false;}
-     int nivellAillamentFitxes=0;
+     solucionaJocActual_CalculaMovimentsPosibles();
+    // 22/08/16 Això no es fa servir per res
+     //int nivellAillamentFitxes=0;
      int numeroFitxesAillades=0;
-     QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+     //QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+     QMapIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
      QString moviments;
+    //posam totes les fitxes a no aïllades
+     //while ( iterator_fitxes.hasNext() ){
+     // iterator_fitxes.next();
+     // iterator_fitxes.value()->setFitxaAillada(false);
+     //}
+     //iterator_fitxes.toFront();
      while ( iterator_fitxes.hasNext() ){
         iterator_fitxes.next();
-        int index_fitxaInicial=iterator_fitxes.value()->coordenades().x()*100+iterator_fitxes.value()->coordenades().y();
+        int index_fitxaInicial=
+            iterator_fitxes.value()->coordenades().x()*100+iterator_fitxes.value()->coordenades().y();
+        int index_fitxaBotada;
+        int index_fitxaFinal;
         moviments.clear();
+        m_fitxes.value(index_fitxaInicial)->calculaMovimentsFitxa();
         //Només poden estar "aïllades" les fitxes actives
-        //(si ho poden estar si son fitxes finals de joc!)
+        //(sí ho poden estar si son fitxes finals de joc!)
         //Tampoc pot estar aïllada una fitxa que és la mitjana d'un moviment posible
-        if( ((m_fitxes.value(index_fitxaInicial)->estat()==1) &&
-            ( !(movimentPosibleFitxaMitjana(index_fitxaInicial))  )) ||
+        if(  (m_fitxes.value(index_fitxaInicial)->estat()==1)   ||
             (!(m_fitxes.value(index_fitxaInicial)->fitxaFinalJoc()) &&
-             ((m_fitxes.value(index_fitxaInicial)->estat()==1)))   ){
+            (m_fitxes.value(index_fitxaInicial)->estat()==1) ) ){
         for(int j=0 ; j<iterator_fitxes.value()->movimentsPossibles.count();++j){
-            int index_fitxaBotada=iterator_fitxes.value()->movimentsPossibles.at(j).x();
-            //fitxa activa amb fitxa mitjana en estat que permet el bot
+            index_fitxaBotada=iterator_fitxes.value()->movimentsPossibles.at(j).x();
+            index_fitxaFinal=iterator_fitxes.value()->movimentsPossibles.at(j).y();
+            //fitxa activa possibilitat de fer un moviment
             if (  (m_fitxes.value(index_fitxaInicial)->estat()==1) &&
-                  (m_fitxes.value(index_fitxaBotada)->estat()==p_estatFitxaMitjana)  ){
-                moviments.append("1"); break;}
+                  (m_fitxes.value(index_fitxaBotada)->estat() ==p_estatFitxaMitjana) &&
+                  (m_fitxes.value(index_fitxaFinal)->estat()==0)){
+                  moviments.append("1"); //break;
+              }
+            else moviments.append("0");
+
             if (  (m_fitxes.value(index_fitxaInicial)->estat()==1) &&
                 (m_fitxes.value(index_fitxaBotada)->estat() != p_estatFitxaMitjana)  ){
                 //Cal mirar si la fitxa botada es el final d'un moviment possible
-                if ( (! movimentPosibleFitxaFinal(index_fitxaBotada)) ){
+                if ( (!movimentPosibleFitxaFinal(index_fitxaBotada)) ){
                        moviments.append("0");
                 }
-                else {moviments.append("1"); break;}
+                else {moviments.append("1");
+                   // break;
+                }
             }
-          }
+            //La fitxa pot ésser la mitjana d'un moviment possible
+            if (  (m_fitxes.value(index_fitxaInicial)->estat()==1)  ){
+                if ( (!movimentPosibleFitxaMitjana(index_fitxaInicial)) ){
+                       moviments.append("0");
+                }
+                else {moviments.append("1");
+                   // break;
+                }
+            }
+            if (  (m_fitxes.value(index_fitxaInicial)->estat()==1) &&
+                  m_fitxes.value(index_fitxaInicial)->movimentsFitxa.count()<1){
+                if ( (!movimentPosibleFitxaMitjana(index_fitxaInicial)) ){
+                       moviments.append("");
+                }
+                else {moviments.append("0");
+                   // break;
+                }
+            }
+          }//final del for
 
         if ( !(moviments.contains("1"))  ){
-            m_fitxes.value(index_fitxaInicial)->setFixaAillada(true);
+            m_fitxes.value(index_fitxaInicial)->setFitxaAillada(true);
             if (m_fitxes.value(index_fitxaInicial)->nivellAillament()==1){
                 numeroFitxesAillades=numeroFitxesAillades+1;}
         }
-        else m_fitxes.value(index_fitxaInicial)->setFixaAillada(false);
-      }
-     else m_fitxes.value(index_fitxaInicial)->setFixaAillada(false);
+        else {
+             m_fitxes.value(index_fitxaInicial)->setFitxaAillada(false);
+        }
+        qApp->processEvents();
+      }//final del if inicial del while
+     else {
+       m_fitxes.value(index_fitxaInicial)->setFitxaAillada(false);
+       }
 
-    nivellAillamentFitxes=nivellAillamentFitxes+m_fitxes.value(index_fitxaInicial)->nivellAillament();
+    //if(m_fitxes.value(index_fitxaInicial)->estat()==1){
+       m_fitxes.value(index_fitxaInicial)->calculaMovimentsFitxa();
+     qDebug("m_fitxes.value(index_fitxaInicial)->movimentsFitxa() %d",
+            m_fitxes.value(index_fitxaInicial)->movimentsFitxa.count());
+     if(m_fitxes.value(index_fitxaInicial)->movimentsFitxa.count()<1){
+          m_fitxes.value(index_fitxaInicial)->setFitxaAillada(true);
+          qDebug("Fitxa sense moviments!********");
+       }
     }
+    if(m_fitxes.value(index_fitxaInicial)->estat()==1){
+        qDebug("m_fitxes.value(index_fitxaInicial)->movimentsFitxa() %d",
+               m_fitxes.value(index_fitxaInicial)->movimentsFitxa.count());
+    }
+
+   // nivellAillamentFitxes=nivellAillamentFitxes+m_fitxes.value(index_fitxaInicial)->nivellAillament();
+    }//final while
+qDebug("p_numFitxesAillades %d", p_numFitxesAillades);
+qDebug("numeroFitxesAillades %d", numeroFitxesAillades);
+//pausa(3000);
 return  (numeroFitxesAillades>p_numFitxesAillades);
+//return  (numeroFitxesAillades>0);
+ }
+ */
+
+ /* Nou procediment per a la versió 2.1
+   Comprova si hi ha alguna fitxa "aïllada"
+   o sigui fitxes en estat 1 que no tenen cap moviment
+   que es pugui fer efectiu o be que no tenen
+   altres fitxes (o buits en el cas de la modialitat inversa)
+   que permetin que es moguin
+   */
+ bool Tauler::solucionaJocActual_FitxesAillades1(){
+     /* Aquest procediment no és eficient
+      * amb la modalitat inversa
+      03/09/16 A la versió 2.1 s'ha fet per la modalitat inversa */
+     //if(p_tipusMoviment==2){return false;}
+     solucionaJocActual_CalculaMovimentsPosibles();
+     /* 22/08/16 Això no es fa servir per res*/
+     //int nivellAillamentFitxes=0;
+     int numeroFitxesAillades=0;
+     //QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+     QMapIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+
+     while ( iterator_fitxes.hasNext() ){
+        iterator_fitxes.next();
+        /* Calcula els moviments actuals (reals) de la fitxa*/
+        iterator_fitxes.value()->calculaMovimentsFitxa();
+        /* Només poden estar "aïllades" les fitxes actives sense moviments.
+         * Poden estar aïllades les fitxes que son fitxes finals de joc*/
+       if(  ((iterator_fitxes.value()->estat()==1) &&
+            (iterator_fitxes.value()->movimentsFitxa.count()==0) ) &&
+            (!(iterator_fitxes.value()->fitxaFinalJoc()))
+            ){
+           /* Es calcula la distància (del taxi) de la fitxa actual a altres fitxes
+           * i també en diagonal si el moviment és en diagonal */
+           QPoint distancies=distanciaDeUnaFitxaAAltres(iterator_fitxes.value()->coordenades());
+           /* distancies.x() fitxes proximes
+            * distancies.y() distància mínima */
+           if(distancies.x() < 2 || distancies.y() > 2 ){
+            iterator_fitxes.value()->setFitxaAillada(true);
+            numeroFitxesAillades=numeroFitxesAillades+1;
+            qApp->processEvents();
+            //pausa(2000);
+            }
+           else iterator_fitxes.value()->setFitxaAillada(false);
+         }//final del if inicial del while
+     else {
+       iterator_fitxes.value()->setFitxaAillada(false);
+       }
+    }//final while
+
+//qDebug("p_numFitxesAillades %d", p_numFitxesAillades);
+//qDebug("numeroFitxesAillades %d", numeroFitxesAillades);
+//pausa(3000);
+return  (numeroFitxesAillades>qMax(3,p_numFitxesAillades));
+//return  (numeroFitxesAillades>qMax(p_numFitxesAillades,1));
+//return  (numeroFitxesAillades>0);
  }
 
+/* Calcula la distància entre una fitxa i les altres per saber
+ * si està aïllada */
+QPoint Tauler::distanciaDeUnaFitxaAAltres(QPoint coordenades){
+QMapIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+int proximitat=3;
+int fitxesProximes=0;
+int distanciaMinima=10000;
+//qDebug("Fitxa %d %d", coordenades.x(),coordenades.y());
+while ( iterator_fitxes.hasNext() ){
+   iterator_fitxes.next();
+   QPoint coordenadesAltraFitxa;
+   bool continuar=false;
+   /* només per fitxes actives*/
+   if(p_tipusMoviment == 2 &&  /* Moviment invers*/
+      iterator_fitxes.value()->estat()==0){
+      coordenadesAltraFitxa=iterator_fitxes.value()->coordenades();
+      continuar=true;
+   }
+   else if (iterator_fitxes.value()->estat()==1){
+   coordenadesAltraFitxa=iterator_fitxes.value()->coordenades();
+   continuar=true;
+    }
+
+   if (continuar){
+   //qDebug("Altra fitxa %d %d", coordenadesAltraFitxa.x(),coordenadesAltraFitxa.y());
+   /* No es calcula la distància d'un fitxa a si mateixa */
+   if(coordenades != coordenadesAltraFitxa){
+   int distancia=abs(coordenades.x()-coordenadesAltraFitxa.x())+
+                 abs(coordenades.y()-coordenadesAltraFitxa.y());
+   if (p_tipusMoviment==3){/* cas de moviments en diagonal
+      * també es calcula les distàncies en diagonal*/
+      if(abs(coordenades.x()-coordenadesAltraFitxa.x())==
+         abs(coordenades.y()-coordenadesAltraFitxa.y()) ){/*Les fitxes estan en la mateixa diagonal*/
+          int distanciaEnDiagonal=abs(coordenades.x()-coordenadesAltraFitxa.x());
+          /* en aquest cas, s'agafa la distància més curta */
+          distancia=qMin(distancia,distanciaEnDiagonal);
+     }
+   }/* if moviment en diagonal 3*/
+   distanciaMinima=qMin(distanciaMinima,distancia);
+   //qDebug("distancia %d",distancia);
+   if(distancia<=proximitat || distanciaMinima < 3){
+     fitxesProximes++;
+     }
+    }/* if de igualtat de coordenades*/
+  }//if continuar
+}/* final del while */
+
+/*qDebug("fitxesProximes %d",fitxesProximes);
+qDebug("distanciaMinima %d",distanciaMinima);*/
+return QPoint(fitxesProximes,distanciaMinima);
+}
 
  /*
    Comprova si hi ha algun moviment posible que tengui com a fitxa final
@@ -835,7 +1112,8 @@ return  (numeroFitxesAillades>p_numFitxesAillades);
    */
  bool Tauler::movimentPosibleFitxaFinal(int fitxaMijana){
   bool resultat=false;
- QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+ QMapIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+ //QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
 
       while (iterator_fitxes.hasNext() && (! resultat) ) {
         iterator_fitxes.next();
@@ -861,9 +1139,11 @@ return  (numeroFitxesAillades>p_numFitxesAillades);
   return resultat;
  }
 
+
  bool Tauler::movimentPosibleFitxaFinal1(int fitxaMijana){
   bool resultat=false;
-  QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+  //QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+  QMapIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
 
       while (iterator_fitxes.hasNext() && (! resultat) ) {
         iterator_fitxes.next();
@@ -882,14 +1162,14 @@ return  (numeroFitxesAillades>p_numFitxesAillades);
   return resultat;
  }
 
-
  /*
    Comprova si hi ha algun moviment possible que tengui la fitxa
    passada per paràmetre com a fitxa botada
    */
  bool Tauler::movimentPosibleFitxaMitjana(int fitxaMitjana){
      bool resultat=false;
-     QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+     //QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+     QMapIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
 
       while (iterator_fitxes.hasNext() && (! resultat)) {
         iterator_fitxes.next();
@@ -936,7 +1216,8 @@ return resultat;
  */
 
  void Tauler::eliminaFitxesVermelles(){
-    QHashIterator <int, Fitxa*> i(m_fitxes);
+    //QHashIterator <int, Fitxa*> i(m_fitxes);
+    QMapIterator <int, Fitxa*> i(m_fitxes);
     while (i.hasNext()) {
         i.next();
         if (i.value()->estat()==5){
@@ -949,7 +1230,8 @@ return resultat;
  * a color blau
  */
  void Tauler::eliminaFitxesVerdes(){
-    QHashIterator <int, Fitxa*> i(m_fitxes);
+    //QHashIterator <int, Fitxa*> i(m_fitxes);
+    QMapIterator <int, Fitxa*> i(m_fitxes);
     while (i.hasNext()) {
         i.next();
         if (i.value()->estat()==4){
@@ -959,28 +1241,37 @@ return resultat;
 }
 
 QString Tauler::configuracioActual(bool comptarFitxes){
-     QHashIterator <int, Fitxa*> i(m_fitxes);
+
+    // QHashIterator <int, Fitxa*> p(m_fitxes);
+     QMapIterator <int, Fitxa*> p(m_fitxes);
      QString configuracioActual;
      if (comptarFitxes){
          comptadorFitxesJoc=0;
          comptadorFitxesEstatZeroJoc=0;
          comptadorFitxesFinalJoc=0;
      }
-        while (i.hasNext()) {
-              i.next();
-              configuracioActual.append(QString("%1").arg((i.value()->estat())));
+
+        int contador=0;
+        while (p.hasNext()) {
+              p.next();
+              configuracioActual.append(QString("%1").arg((p.value()->estat())));
+              //qCritical("configuracioActual %s",qPrintable(configuracioActual));
+              //qDebug("Key %d estat %d",p.key(),p.value()->estat());
+
               if (comptarFitxes){
-               if (i.value()->estat()==1){++comptadorFitxesJoc;}
-               if (i.value()->estat()==0){++comptadorFitxesEstatZeroJoc;}
-               if (i.value()->fitxaFinalJoc()){
+               if (p.value()->estat()==1){++comptadorFitxesJoc;}
+               if (p.value()->estat()==0){++comptadorFitxesEstatZeroJoc;}
+               if (p.value()->fitxaFinalJoc()){
                   ++comptadorFitxesFinalJoc;
                   }
               }
+           ++contador;
           }
 /* Això cal quan es modifica un joc del programa
  * i s'han eliminat fitxes.
  */
 configuracioActual.remove("6");
+//qDebug("contador %d",contador);
 //qCritical("configuracioActual %s",qPrintable(configuracioActual));
 return configuracioActual;
  }
@@ -1005,7 +1296,8 @@ return configuracioActual;
 
  //Carrega la configuració passada
 void Tauler::carregaConfiguracio(QString configuracio){
-     QHashIterator <int, Fitxa*> i(m_fitxes);
+     //QHashIterator <int, Fitxa*> i(m_fitxes);
+     QMapIterator <int, Fitxa*> i(m_fitxes);
      int contador=0;
      while (i.hasNext()) {
               i.next();
@@ -1021,7 +1313,8 @@ Es fa servir quan el joc està en pausa
 */
 void Tauler::estatFitxesAtzar(){
     qsrand(QTime::currentTime().msec());
-     QHashIterator <int, Fitxa*> i(m_fitxes);
+     //QHashIterator <int, Fitxa*> i(m_fitxes);
+     QMapIterator <int, Fitxa*> i(m_fitxes);
      while (i.hasNext()) {
               i.next();
               i.value()->setEstat(qrand() % 5);
@@ -1097,7 +1390,8 @@ void Tauler::estatFitxesAtzar(){
   * joc generat a l'atzar
   */
  QString Tauler::solitariAtzar_dadesEstatFitxes(int files, int columnes){
-     QHashIterator <int, Fitxa*> i(m_fitxes);
+     //QHashIterator <int, Fitxa*> i(m_fitxes);
+     QMapIterator <int, Fitxa*> i(m_fitxes);
      QString dadesFitxes;
      QStringList dadesFitxesList;
      //omplim la llista
@@ -1160,7 +1454,8 @@ void Tauler::estatFitxesAtzar(){
  for(int j=0; j<files*columnes; j++){
      dadesFitxesList.append("-1");
  }
- QHashIterator <int, Fitxa*> i(m_fitxes);
+ //QHashIterator <int, Fitxa*> i(m_fitxes);
+ QMapIterator <int, Fitxa*> i(m_fitxes);
  i.toFront();
  while (i.hasNext()) {
           i.next();
@@ -1203,7 +1498,8 @@ void Tauler::estatFitxesAtzar(){
   * de files i columnes.
   */
  QString Tauler::solitariPersonalitzat_filesColumnes(){
-  QHashIterator <int, Fitxa*> i(m_fitxes);
+  //QHashIterator <int, Fitxa*> i(m_fitxes);
+  QMapIterator <int, Fitxa*> i(m_fitxes);
   int minFila=1000, maxFila=0,minCol=1000,maxCol=0;
   while (i.hasNext()) {
      i.next();
@@ -1239,7 +1535,8 @@ void Tauler::estatFitxesAtzar(){
   * (segons el valor passat)
   */
  void Tauler::marcaFitxesSolitariPersonalitzat(bool valor){
-     QHashIterator <int, Fitxa*> i(m_fitxes);
+     QMapIterator <int, Fitxa*> i(m_fitxes);
+     //QHashIterator <int, Fitxa*> i(m_fitxes);
      while (i.hasNext()) {
         i.next();
         i.value()->setFitxaSolitariPersonalitzat(valor);
