@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2010-2013 Innocent De Marchi <tangram.peces@gmail.com>
+ * Copyright (C) 2010-2016 Innocent De Marchi <tangram.peces@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,8 @@
 #ifndef TAULER_H
 #define TAULER_H
 
-#include <QGraphicsView>
+#include <QtWidgets/QGraphicsView>
+#include <QtWidgets/QUndoStack>
 #include <QHash>
 #include <QtGui>
 #include "constants.h"
@@ -30,8 +31,9 @@ class Fitxa;
 
 class Tauler: public QGraphicsView
 {
+    Q_GADGET
 public:
-    Tauler( QUndoStack  *movimentsUndoStack, Rellotge *rellotge,frmPrincipal *frmPrinci, QWidget* parent = 0);
+    Tauler( QUndoStack  *movimentsUndoStack, Rellotge *rellotge,frmPrincipal *frmPrinci, QWidget *parent = 0);
 
     //QGraphicsScene* sceneJoc;
 
@@ -49,7 +51,8 @@ public:
     void marcaMoviment (QString moviment);
 
 
-    QHash<int, Fitxa*> m_fitxes;
+    QMap<int, Fitxa*> m_fitxes;
+    //QHash<int, Fitxa*> m_fitxes;
 
     QUndoStack  *p_movimentsUndoStack;
 
@@ -75,6 +78,8 @@ public:
        (QStringList movimentsFets, QString darrerMoviment);
 
     QStringList solucionaJocActual_CalculaNumeroMovimentsPosteriors(QStringList movimentsPosibles);
+    QStringList solucionaJocActual_SeleccionaNumeroMovimentsPosteriors(QStringList llistaMovimentsMaxims,
+                                                                      QList<int> llistaNumeroMoviments);
     /* Comprova si, desprès de fer el moviment passat per paràmetre,
      * és possible fer altres moviments
      * 20/11/12 S'ha eliminat: no és eficaç
@@ -88,10 +93,10 @@ public:
       o sigui fitxes en estat 1 que no tenen cap moviment
       que es pugui fer efectiu o be cap moviment en què
       la fitxa mitjancera pugui passar a estat 1 desprès d'un moviment
-      El proc. no és eficaç en les modalitats inverses
-      (i no es fa servir en aquest cas)
       */
-    bool solucionaJocActual_FitxesAillades();
+    //bool solucionaJocActual_FitxesAillades(); eliminat a la versió 2.1
+    bool solucionaJocActual_FitxesAillades1();
+    QPoint distanciaDeUnaFitxaAAltres(QPoint coordenades);
     bool movimentPosibleFitxaFinal(int fitxaMijana);
     bool movimentPosibleFitxaFinal1(int fitxaMijana);
     bool movimentPosibleFitxaMitjana(int fitxaMitjana);
@@ -141,7 +146,9 @@ public:
     //a partir d'una altra
     int p_numMoviments;
 
-    //Tipus de moviment del joc actual
+    /* Tipus de moviment del joc actual
+     *  1.directe 2.invers 3. Diagonal
+     */
     int p_tipusMoviment;
 
     /*
@@ -215,6 +222,7 @@ public:
 
     frmPrincipal *p_frmPrinci;
 
+    bool jocFinalitzat();
 
  protected:
     virtual void resizeEvent(QResizeEvent* event);
@@ -227,7 +235,6 @@ public:
     QString p_modalitatJocActual;
     int p_estatFitxaMitjana;
 
-    bool jocFinalitzat();
     bool nomesQuedenFitxesMarcadesFinalJoc();
     /* Comprova si s'ha arribat al final del joc. Això
      * és molt útil en les modalitats que tenen el final
